!integrateRanges
! Module: colimplot_common
!
! Authors:  Ian Stewart - SSC-LUX
!
! Description: Contains declarations for common colimplot variables.
!
! Comments:
!
! $Id: test_routines.f90,v 1.6 1998/05/27 13:45:39 gvacanti Exp $
!- 
module colimplot_common

! You should write the module name in the 'use' statement, the package name
! (and version number) in the DEPEND file, and the * from the lib*.a name in
! <package>/src/Makefile.
  use dal
  use caloaldefs
  use cal
  use errorHandling
  use dss

  use linear	! for fitLine.
!  use dss_aux	! for integrateRanges.
  use dss_ranges_aux	! for integrateRanges.
  use constants
  use ssc_misc	! for fracVal
  use warp_utils	! for warp, biWarp

  implicit none 

  real(single), dimension(:), pointer, public ::&
    bbRefColour, imageBandCentres, multiplet
  real(single), parameter, public ::&
!    plancksConst = 6.6260755e-34,&
!    mySpeedOfLight = 2.99792458e8,&
!    wienConst = 2.897756e-3,&
!    myElectronQ = 1.60217733e-19,&
!    boltzmannConst = 1.380658e-23,&
!    qeOnBoltzmann = myElectronQ / boltzmannConst,&
!qeOnBoltzmann = ElectronQ / Boltzmann,&
    bbPeakConst = 2.8214394
  real(single), public ::&
    weirdness = 0.0,&
    heat = 0.0,&
    heatSpread = 0.0,&
    dE,&
    averageBandCentre,&
    clrGainDistortion

  real(kind=double), parameter, public ::&
    minEeV = 0.0d0,&
    maxEeV = 10000.0d0
  real(single), dimension(8), parameter, public ::&
    bandLoEeV = (/200.0, 500.0, 2000.0, 4500.0, 7500.0, 200.0, 2000.0, 100.0/),&
    bandHiEeV = (/500.0, 2000.0, 4500.0, 7500.0, 12000.0, 2000.0, 12000.0, 12000.0/)
!***** check these against current products!
!**** These should be stored centrally - in the cal?

  integer(int16), parameter, public ::&
    red = 1,&
    grn = 2,&
    blu = 3,&
    numEbins = 1000
  integer(int16), public ::&
    numSuppliedBands
  integer, public ::&
    minClrIndex,&
    maxClrIndex,&
    maxNumClrIndices
!  integer, parameter :: OPEN = 1	! From dssutil. Check that the
!  integer, parameter :: CLOSED = 2	! match-up is as I have guessed it.
!  integer, parameter, public :: INCLUSIVE = 1
!  integer, parameter, public :: EXCLUSIVE = 2
!  integer, parameter, public :: UNDEFINED = 3
  real(single), public :: refThermIndex(-100:100), refThermRGB(-100:100, red:blu)

  integer(int32), public ::&
    instrument,&
    OMfilter

  type, public :: treeType
    real(single), dimension(3) ::&
      min,&
      max
    logical ::&
      empty,&
      isLeaf
    integer ::&
      clrIndex
  end type
!  type, public :: boundType
!    integer :: type
!    real(kind(1d0)) :: value
!  end type boundType
!  type, public :: intervalType
!    type(boundType) :: lower, upper
!  end type intervalType
!  type, public :: intervalsType
!    type(boundType) :: lower, upper
!    integer :: index
!  end type intervalsType
!  type, public :: sequenceType
!    type(boundType) :: bound
!    logical         :: isLower
!    integer         :: index
!  end type sequenceType

  private 

  character(3) :: clrStr(red:blu) = (/'red', 'grn', 'blu'/)

  public integrateSpectrum
  public calcOMfilterTrans
  public getThermalClrAndPurplicity
  public getThermalIndices
  public calcBandCentre
  public getColourSaturatedImage
  public implementWeirdness
!  public lobias
!  public spread

contains 

  !---------------------------------------------------------------------
!  function integrateSpectrum(energyIntervals, tempMegaKelvin) result(realSumm)
function integrateSpectrum_20031027(energyRanges, tempMegaKelvin) result(realSumm)
  !---------------------------------------------------------------------
  ! NOTE! Requires call to CAL_setState beforehand.

!*** can use trapezoid rule integration in ssclib??

    implicit none

!    type(intervalType), intent(in) :: energyIntervals(:)
type(RangeT), intent(in) :: energyRanges(:)
    real, intent(in) :: tempMegaKelvin
    integer(int16) :: numBins, bin, loN, hiN, i
    real :: dElocal, realSumm
real(double) :: botEeV, topEeV, summ, photonEeV, kelvin
!, hertz
    real, allocatable :: inInterval(:)
    real(double), allocatable :: hertzValues(:), bbPowers(:)

    !-------------------------------------------------------------------
    ! Start:

    kelvin = tempMegaKelvin * 1.0d6

!    botEeV = minval(energyIntervals%lower%value)
!    topEeV = maxval(energyIntervals%upper%value)
botEeV = minval(energyRanges%lower%value)
topEeV = maxval(energyRanges%upper%value)
!*** error if lower or upper type is UNDEFINED?

    numBins = 100
!***** something more general or global required?

    allocate(inInterval(numBins), hertzValues(numBins), bbPowers(numBins))
    inInterval = 0.0 ! vector default
!    do i = 1, size(energyIntervals)
do i = 1, size(energyRanges)
!      loN = 1 + nint(numBins * ((energyIntervals(i)%lower%value - botEeV)&
loN = 1 + nint(numBins * ((energyRanges(i)%lower%value - botEeV)&
      / (topEeV - botEeV)))
!      hiN = 1 + nint(numBins * ((energyIntervals(i)%upper%value - botEeV)&
hiN = 1 + nint(numBins * ((energyRanges(i)%upper%value - botEeV)&
      / (topEeV - botEeV)))
      loN = min(loN, numBins)
      hiN = min(hiN, numBins)
      inInterval(loN:hiN) = 1.0
    end do

    dElocal = (topEeV - botEeV) / real(numBins)

    do bin = 1, numBins
      photonEeV = (botEeV + (bin - 0.5) * dElocal)
      hertzValues(bin) = SpeedOfLight / (eV2angstroms(photonEeV) * 1.0e-10)
    end do
    call getBBfluxPerHertz(hertzValues, kelvin, bbPowers)

    summ = 0.0
    if (instrument .eq. EMOS1 .or. instrument .eq. EMOS2&
    .or. instrument .eq. EPN) then
      do bin = 1, numBins
        photonEeV = (botEeV + (bin - 0.5) * dElocal)
!        hertz = SpeedOfLight / (eV2angstroms(photonEeV) * 1.0e-10)
!        summ = summ + bbFluxPerHertz(hertz, kelvin)&
        summ = summ + bbPowers(bin)&
        * CAL_getEffectiveArea(photonEeV, 0.0d0, 0.0d0)
!print *, bin, photonEeV, bbPowers(bin)&
!, CAL_getEffectiveArea(photonEeV, 0.0d0, 0.0d0), summ
      end do

    else ! instrument should be OM. Others should be screened out in
    ! subroutine inputChecks.
      do bin = 1, numBins
        photonEeV = (botEeV + (bin - 0.5) * dElocal)
!        hertz = SpeedOfLight / (eV2angstroms(photonEeV) * 1.0e-10)
!        summ = summ + bbFluxPerHertz(hertz, kelvin)&
        summ = summ + bbPowers(bin)&
        * calcOMfilterTrans(OMfilter, eV2angstroms(photonEeV))
      end do
!*** an error here the same as in inputChecks wouldn't do any harm.
    end if
    realSumm = real(summ * dElocal)

!print *, ''
!print *, '       Inside integrateSpectrum: temp = ', kelvin
!print *, '       Inside integrateSpectrum: result = ', realSumm

  end function integrateSpectrum_20031027

  !---------------------------------------------------------------------
  function integrateSpectrum(EeVRanges, tempMegaKelvin) result(realSumm)
  !---------------------------------------------------------------------
  ! NOTE! (i) Requires call to CAL_setState beforehand. (ii) Assumes that EeVRanges is sorted in increasing order.

    implicit none

    type(RangeT), intent(in) :: EeVRanges(:)
    real(single), intent(in) :: tempMegaKelvin
    real(single) :: realSumm

    integer(int16) :: numSamples, sample!, i
    real(double) :: botEeV, topEeV, kelvin
    real(double), allocatable :: hertzValues(:), bbPowers(:), photonEeV(:)
    real(single), allocatable :: spectrum(:)
!real(single), allocatable :: frac(:)
integer :: i

    !-------------------------------------------------------------------
    ! Start:

    kelvin = tempMegaKelvin * 1.0d6

    botEeV = minval(EeVRanges%lower%value)
    topEeV = maxval(EeVRanges%upper%value)
!*** error if lower or upper type is UNDEFINED?
!*** error if topEeV <= botEeV?

    numSamples = 100
!***** something more general or global required?

    allocate(photonEeV(0:numSamples), hertzValues(0:numSamples))
    allocate(bbPowers(0:numSamples), spectrum(0:numSamples))
!allocate(frac(0:numSamples))
!frac = (/(i/real(numSamples,single), i=0,numSamples)/)
!oldXedges = (/(real(i, single), i=1,oldXsize)/)
!frac = (/(i, i=0,numSamples)/)
!    photonEeV = fracVal(real(botEeV, single), real(topEeV, single), frac) ! in ssclib/array_utils
    photonEeV = fracVal(botEeV, topEeV&
      , (/(i/real(numSamples,double), i=0,numSamples)/)) ! in ssclib/ssc_misc

    do sample = 0, numSamples
      hertzValues(sample) = SpeedOfLight / (eV2angstroms(photonEeV(sample))&
        * 1.0e-10) ! eV2angstroms is in ssclib/constants
    end do
    call getBBfluxPerHertz(hertzValues, kelvin, bbPowers) ! in ssclib/constants

!    summ = 0.0
    if (instrument .eq. EMOS1 .or. instrument .eq. EMOS2&
    .or. instrument .eq. EPN) then
      do sample = 0, numSamples
        spectrum(sample) = bbPowers(sample)&
          * CAL_getEffectiveArea(photonEeV(sample), 0.0d0, 0.0d0)
      end do

    else ! instrument should be OM. Others should be screened out in
    ! subroutine inputChecks.
      do sample = 0, numSamples
        spectrum(sample) = bbPowers(sample)&
          * calcOMfilterTrans(OMfilter, eV2angstroms(photonEeV(sample))) ! eV2angstroms is in ssclib/constants
      end do
!*** an error here the same as in inputChecks wouldn't do any harm.
    end if

    realSumm = integrateRanges(real(photonEeV, single), spectrum&
      , EeVRanges) ! in ssclib/dss_ranges_aux

  end function integrateSpectrum

  !---------------------------------------------------------------------
  function calcOMfilterTrans(OMfilter, angstroms) result(effArea)
  !---------------------------------------------------------------------
!*** At the moment I have no way to calculate a good approximation to the transmission curves of these OM filters. So at present I am modelling them by top-hat bandpasses and therefore give a very approximate average value valid for the whole width of each filter.
    implicit none

    real(kind=double), intent(in) :: angstroms
    integer(kind=int32), intent(in) :: OMfilter
    real(kind=double) :: effArea
real(kind=double) :: dummy

    !-------------------------------------------------------------------
    ! Start:

dummy = angstroms ! just to avoid a compiler warning about an unused argument.

    select case(OMfilter)
    case (UVW2)
      effArea = 8.0d0
    case (UVM2)
      effArea = 13.0d0
    case (UVW1)
      effArea = 20.0d0
    case (U)
      effArea = 55.0d0
    case (B)
      effArea = 45.0d0
    case (V)
      effArea = 18.0d0
    case (WHITE)
      effArea = 40.0d0
    case default
call error('badOMfilter', "Can't calculate a bandpass for this OM &
&filter setting. Use refclrstyle=auto.")
    end select

  end function calcOMfilterTrans

  !---------------------------------------------------------------------
  subroutine getThermalIndices(pixelVals, average, thermIndex, purpIndex)
  !---------------------------------------------------------------------
  ! Requires the following:
  !	allocate(imageBandCentres(<number of bands>)&
  !	, multiplet(<number of bands>))
  !	averageBandCentre = sum(imageBandCentres) / real(<number of bands>)

    implicit none

    real(single), intent(in)            :: pixelVals(:)
    real(single), intent(out), optional :: average, thermIndex, purpIndex

    integer(int16) :: numBands
!    real :: intercept, slope, coeffs(3)
real(single) :: intercept, slope, coeffs(3)
    real(single) :: localThermIndex, localAverage, localPurpIndex
    integer :: status

    !-------------------------------------------------------------------
    ! Start

!print *, '  getThermalIndices: start'
    numBands = size(pixelVals)
!print *, '  getThermalIndices: numBands:', numBands
!**** have some errors on size of numBands

    localAverage = sum(pixelVals) / real(numBands)
    if (numBands .gt. 1) then
!print *, '  000'
      multiplet = pixelVals - localAverage
!print *, '  111'
      call fitLine(imageBandCentres, multiplet, intercept, slope, status=status)
!print *, '  222'
      if (status .eq. -3) then
call error('badLinfitExit', 'Thermal index could not be calculated.')
      else if (status .lt. 0) then
call error('bug101', '')
      end if
!print *, '  333'

!**** errors on status values
!print *, 'intercept, slope, status:', intercept, slope, status
!print *, 'imageBandCentres:', imageBandCentres
!print *, 'multiplet:', multiplet
      if (localAverage .gt. 0.0) then
        localThermIndex = slope * abs(averageBandCentre / localAverage)
      else
        localThermIndex = 0.0
      end if
!print *, '  444'

      if (numBands .gt. 2) then
        multiplet = multiplet - intercept - slope * imageBandCentres
!print *, '  555'
!print *, 'sizes of imageBandCentres, multiplet, coeffs:'&
!, size(imageBandCentres), size(multiplet), size(coeffs) 
        call fitPolynomial(imageBandCentres, multiplet, coeffs, status=status)
        ! in ssclib/linear
!**** errors on status values
!print *, '  666'

        if (localAverage .gt. 0.0) then
          localPurpIndex = coeffs(3) * abs(averageBandCentre&
          * averageBandCentre / localAverage)
        else
          localPurpIndex = 0.0
        end if
!print *, '  777'

      else
        localPurpIndex = 0.0
      end if
!print *, 'localAverage, localThermIndex, localPurpIndex:', localAverage, localThermIndex, localPurpIndex
    else
      localThermIndex = 0.0
      localPurpIndex = 0.0
    end if

!print *, '  888'
    if (present(average))    average    = localAverage
    if (present(thermIndex)) thermIndex = localThermIndex
    if (present(purpIndex))  purpIndex  = localPurpIndex
!print *, '  getThermalIndices: end'

  end subroutine getThermalIndices

  !---------------------------------------------------------------------
  subroutine getColourSaturatedImage(images, rgbImage)
  !---------------------------------------------------------------------

    implicit none

    real(single), dimension(:,:,:), pointer :: images, rgbImage
!    integer(int16), intent(in) :: xsize, ysize
!    real(single), intent(out) ::&
!      rgbImage(xsize, ysize, red:blu)

    real(single), allocatable :: tempPixelVals(:)
    real(single), allocatable :: therVals(:,:), purpVals(:,:)
    integer(int16) :: xsize, ysize, numBands, xi, yi, i
    real(single) :: thermIndex, purpIndex, maxThermalIndex, maxPurpIndex,&
!      tempArray(xsize, ysize),&
!      unnormalisedThermIndex, unnormalisedPurpIndex,&
      frac
!integer :: devId, plotStatus
    integer :: foreignStatus

    !-------------------------------------------------------------------
    ! Start:

!print *, 'getColourSaturatedImage: start'
    call message(AppMsg, Verbose, 'Transforming colour info.')

    numBands = size(images, 1)
    xsize = size(images, 2)
    ysize = size(images, 3)
!print *, 'getColourSaturatedImage: sizes 1,2,3 of image:', numBands, xsize, ysize

    allocate(tempPixelVals(numBands))
!print *, 'aaa'
    tempPixelVals = 0.0
!print *, 'bbb'
    tempPixelVals(numBands) = 1.0
!print *, 'ccc'
    call getThermalIndices(tempPixelVals, thermIndex=maxThermalIndex)
!    tempPixelVals = 0.0
!    tempPixelVals(1) = 1.0
!    tempPixelVals(size(images, 1)) = 1.0
!    call getThermalIndices(tempPixelVals, purpIndex=maxPurpIndex)
!print *, 'ddd'
    deallocate(tempPixelVals)

!print *, 'eee'

    allocate(therVals(xsize, ysize), purpVals(xsize, ysize))
    if (numBands .lt. 2) then
      therVals = 0.0
      purpVals = 0.0

    else
      do xi = 1, xsize
        do yi = 1, ysize
          call getThermalIndices(images(:, xi, yi)&
          , thermIndex=therVals(xi, yi), purpIndex=purpVals(xi, yi))
        end do
      end do
!      maxThermalIndex = max(maxval(therVals), -minval(therVals))
      maxPurpIndex    = max(maxval(purpVals), -minval(purpVals))
      if (maxThermalIndex .gt. 0.0) therVals = therVals / maxThermalIndex
where(therVals .gt. 1.0) therVals = 1.0
where(therVals .lt. -1.0) therVals = -1.0
      if (maxPurpIndex    .gt. 0.0) purpVals = purpVals / maxPurpIndex
    end if

    deallocate(images)
    allocate(rgbImage(xsize, ysize, red:blu))

!print *, 'fff'

    do xi = 1, xsize
      do yi = 1, ysize
        thermIndex = min(1.0, max(-1.0, therVals(xi, yi)))
!        thermIndex = lobias(heat, (thermIndex + 1.0) / 2.0) * 2.0 - 1.0
call warp(heat, (thermIndex + 1.0) / 2.0, thermIndex, foreignStatus) ! in ssclib/warp_utils
!*** should test foreignStatus!
thermIndex = thermIndex * 2.0 - 1.0
!        thermIndex = spread(heatSpread, thermIndex)
call biWarp(heatSpread, thermIndex, thermIndex, foreignStatus) ! in ssclib/warp_utils
!*** should test foreignStatus!

        purpIndex = min(1.0, max(-1.0, purpVals(xi, yi)))
!        purpIndex = spread(weirdness, purpIndex)
call biWarp(weirdness, purpIndex, purpIndex, foreignStatus) ! in ssclib/warp_utils
!*** should test foreignStatus!

        i = min(floor(100 * thermIndex), 100)
        if (i .eq. 100) then
          rgbImage(xi, yi, :) = refThermRGB(100, :)
        else if (i .lt. -100) then
          rgbImage(xi, yi, :) = refThermRGB(-100, :)
        else
          frac = 100 * thermIndex - floor(100 * thermIndex)
          rgbImage(xi, yi, :) = refThermRGB(i, :) * (1.0 - frac)&
          + refThermRGB(i + 1, :) * frac
        end if

        call implementWeirdness(purpIndex, rgbImage(xi, yi, :))

        rgbImage(xi, yi, :) = rgbImage(xi, yi, :) / sum(rgbImage(xi, yi, :))
      end do
    end do
    deallocate(therVals, purpVals)

    clrGainDistortion = maxval(rgbImage)
    rgbImage = rgbImage / clrGainDistortion

!print *, 'zzz'
!print *, 'getColourSaturatedImage: end'

  end subroutine getColourSaturatedImage

  !---------------------------------------------------------------------
  subroutine implementWeirdness(purpIndex, rgb)
  !---------------------------------------------------------------------

    implicit none

    real(single), intent(in) :: purpIndex
    real(single), intent(inout) :: rgb(red:blu)

    real(single) :: x, y
    integer :: mi(1)
logical :: flag = .false.

    !-------------------------------------------------------------------
    ! Start

    mi = maxloc(rgb)

if (maxval(rgb) .ne. 1.0) call error('bug99','')

if (flag) then
print *, ''
print *, 'starting implementWeirdness.'
print *, 'brightest colour: ', clrStr(mi(1))
end if

    if (purpIndex .ge. 0.0) then
if (flag) print *, 'purpIndex', purpIndex, '.ge. 0.0'
      if (mi(1) .eq. red) then
        x = 1.0 - rgb(blu)
        y = -rgb(grn)

      else if (mi(1) .eq. grn) then
        x = rgb(red) - 1.0
        y = rgb(blu) - 2.0

      else ! mi(1) .eq. blu
        x = rgb(red) - 1.0
        y = -rgb(grn)
      end if

if (flag) print *, 'rgb', rgb
if (flag) print *, 'x, y:', x, y

      x = x * (1.0 - purpIndex)
      y = y * (1.0 - purpIndex)

if (flag) print *, 'new x, y:', x, y

      if (x .gt. 0.0) then ! red
        rgb(red) = 1.0
        rgb(grn) = -y
        rgb(blu) = 1.0 - x

      else if (y .lt. -1.0) then ! grn
        rgb(red) = 1.0 + x
        rgb(grn) = 1.0
        rgb(blu) = 2.0 + y

      else ! blu
        rgb(red) = 1.0 + x
        rgb(grn) = -y
        rgb(blu) = 1.0
      end if
if (flag) print *, 'new rgb', rgb

    else ! purpIndex < 0:
if (flag) print *, 'purpIndex', purpIndex, '.lt. 0.0'
      if (mi(1) .eq. red) then
        x = -rgb(blu)
        y = 2.0 - rgb(grn)

      else if (mi(1) .eq. grn) then
        x = -rgb(blu)
        y = rgb(red)

      else ! mi(1) .eq. blu
        x = -2.0 + rgb(grn)
        y = rgb(red)
      end if

if (flag) print *, 'rgb', rgb
if (flag) print *, 'x, y:', x, y

      x = x * (1.0 + purpIndex)
      y = y * (1.0 + purpIndex)

if (flag) print *, 'new x, y:', x, y

      if (y .gt. 1.0) then ! red
        rgb(red) = 1.0
        rgb(grn) = 2.0 - y
        rgb(blu) = -x

      else if (x .ge. -1.0) then ! grn
        rgb(red) = y
        rgb(grn) = 1.0
        rgb(blu) = -x

      else ! blu
        rgb(red) = y
        rgb(grn) = x + 2.0
        rgb(blu) = 1.0
      end if
if (flag) print *, 'new rgb', rgb
    end if

if (maxval(rgb) .ne. 1.0) call error('bug100','')

if (flag) print *, 'ending implementWeirdness.'

  end subroutine implementWeirdness

  !---------------------------------------------------------------------
  subroutine getThermalClrAndPurplicity(pixelVals, average, thermalClr&
  , purplicity)
  !---------------------------------------------------------------------
  ! Requires the following:
  !	allocate(imageBandCentres(<number of bands>)&
  !	, multiplet(<number of bands>))
  !	averageBandCentre = sum(imageBandCentres) / real(<number of bands>)

    implicit none

    real(single), intent(in) :: pixelVals(:)
    real(single), intent(out) :: average, thermalClr, purplicity

    integer(int16) :: numBands
!    real :: intercept, slope, coeffs(3), thermIndex, purpIndex
real :: thermIndex, purpIndex
real(single) :: intercept, slope, coeffs(3)
    integer :: status

    !-------------------------------------------------------------------
    ! Start

    numBands = size(pixelVals)
!**** have some errors on size of numBands

    average = sum(pixelVals) / real(numBands)
    if (numBands .gt. 1) then
      multiplet = pixelVals - average
      call fitLine(imageBandCentres, multiplet, intercept, slope, status=status)
!**** errors on status values
!print *, imageBandCentres
!print *, multiplet
!print *, intercept, slope
!print *, ''

      if (average .gt. 0.0) then
        thermIndex = slope * abs(averageBandCentre / average)
!print *, averageBandCentre, average
!print *, 'te', thermIndex
      else
        thermIndex = 0.0
      end if

      if (thermIndex .ge. 0.0) then
        thermalClr = (2.0 + 3.0 * thermIndex) / (1.0 + thermIndex)
      else
        thermalClr = 4.0 / (2.0 - thermIndex)
      end if
!print *, 'tc', thermalClr
!call error('','')

      if (numBands .gt. 2) then
        multiplet = multiplet - intercept - slope * imageBandCentres
        call fitPolynomial(imageBandCentres, multiplet, coeffs, status=status)
!**** errors on status values

        if (average .gt. 0.0) then
          purpIndex = coeffs(3) * abs(averageBandCentre * averageBandCentre&
          / average)
        else
          purpIndex = 0.0
        end if

        purplicity = purpIndex / (1.0 + abs(purpIndex))
      else
        purplicity = 0.0
      end if
    else
      thermalClr = 2.0
      purplicity = 0.0
    end if

  end subroutine getThermalClrAndPurplicity

  !---------------------------------------------------------------------
!  subroutine calcBandCentre(energyIntervals, imageBandCentre)
subroutine calcBandCentre(energyRanges, imageBandCentre)
  !---------------------------------------------------------------------

    implicit none

!    type(intervalType), intent(in) :: energyIntervals(:)
type(RangeT), intent(in) :: energyRanges(:)
    real(single), intent(out) :: imageBandCentre

    integer(int16) :: numBins, bin, loN, hiN, i
    real(single) :: botEeV, topEeV, dElocal, sum1, sum2
    real(double) :: photonEeV
    real(single), allocatable :: inInterval(:)

    !-------------------------------------------------------------------
    ! Start:

!    botEeV = minval(energyIntervals%lower%value)
!    topEeV = maxval(energyIntervals%upper%value)
botEeV = minval(energyRanges%lower%value)
topEeV = maxval(energyRanges%upper%value)
!*** error if lower or upper type is UNDEFINED?

    numBins = 100
!***** something more general or global required?

    allocate(inInterval(numBins))
    inInterval = 0.0 ! vector default
!    do i = 1, size(energyIntervals)
do i = 1, size(energyRanges)
!      loN = 1 + nint(numBins * ((energyIntervals(i)%lower%value - botEeV)&
loN = 1 + nint(numBins * ((energyRanges(i)%lower%value - botEeV)&
      / (topEeV - botEeV)))
!      hiN = 1 + nint(numBins * ((energyIntervals(i)%upper%value - botEeV)&
hiN = 1 + nint(numBins * ((energyRanges(i)%upper%value - botEeV)&
      / (topEeV - botEeV)))
      loN = min(loN, numBins)
      hiN = min(hiN, numBins)
      inInterval(loN:hiN) = 1.0
    end do

    dElocal = (topEeV - botEeV) / real(numBins)
    sum1 = 0.0d0
    sum2 = 0.0d0
    if (instrument .eq. EMOS1 .or. instrument .eq. EMOS2&
    .or. instrument .eq. EPN) then
      do bin = 1, numBins
        photonEeV = (botEeV + (bin - 0.5) * dElocal)
        sum1 = sum1 + photonEeV * CAL_getEffectiveArea(photonEeV, 0.0d0, 0.0d0)
        sum2 = sum2 + CAL_getEffectiveArea(photonEeV, 0.0d0, 0.0d0)
      end do

    else ! instrument should be OM. Others should be screened out in
    ! subroutine inputChecks.
      do bin = 1, numBins
        photonEeV = (botEeV + (bin - 0.5) * dElocal)
        sum1 = sum1 + photonEeV&
        * calcOMfilterTrans(OMfilter, eV2angstroms(photonEeV))
        sum2 = sum2 + calcOMfilterTrans(OMfilter, eV2angstroms(photonEeV))
      end do
!*** an error here the same as in inputChecks wouldn't do any harm.
    end if
    imageBandCentre = sum1 / sum2

  end subroutine calcBandCentre

end module colimplot_common

